# IoC Annotation

Quickly generate IoC singleton objects through annotations.

## Installation

```yaml
# pubspec.yaml
dependencies:
  ioc_annotation:

dev_dependencies:
  build_runner:
```

## Usage

1. Use `@IoC` to mark classes that need to be instantiated.

```dart
// person.dart
abstract class Person {
  hello();
}

// Student.dart
import 'package:ioc_annotation/annotation.dart';
import 'package:test_demo/person.dart';

@IoC()
class Student implements Person {
  @override
  hello() {
    print('hello');
  }
}
```

2. Use `@IoCInstance` to mark the single instance that needs to be generated.

```dart
// services.dart
import 'package:ioc_annotation/annotation.dart';

@IoCInstance()
abstract class Services {}
```

3. Execute this command.

```sh
flutter pub run build_runner build
```

4. Automatically generated IoC instances.

```dart
// services.ioc.dart
class Services {
  /// ....
}
```

## LICENSE

[MIT](LICENSE)
