import 'package:analyzer/dart/element/element.dart';
import 'package:ioc_annotation/ioc_annotation.dart';
import 'package:source_gen/source_gen.dart';
import 'package:build/build.dart';

List<String> importList = List.of([]);
Map<String, String> injectMap = Map.of({});

class IoCGenerator extends GeneratorForAnnotation<IoC> {
  @override
  generateForAnnotatedElement(
    Element element,
    ConstantReader annotation,
    BuildStep buildStep,
  ) {
    importList.add('import \'${buildStep.inputId.uri.toString()}\';');

    if (element.kind != ElementKind.CLASS) {
      return;
    }

    (element as ClassElement).allSupertypes.forEach((item) {
      if (item.superclass?.element.kind == ElementKind.CLASS) {
        final importUri = 'import \'${item.element.location?.components[0]}\';';
        if (importList.indexOf(importUri) < 0) {
          importList.add(importUri);
        }
        injectMap[item.element.displayName] = element.displayName;
      }
    });
  }
}

class IoCInstanceGenerator extends GeneratorForAnnotation<IoCInstance> {
  @override
  generateForAnnotatedElement(
    Element element,
    ConstantReader annotation,
    BuildStep buildStep,
  ) {
    print(injectMap);
    final className = element.name;

    String injectString = '';

    injectMap.forEach((key, value) {
      injectString += '''
        $key _${key.toLowerCase()} = new $value();

        $key get$key() {
          return _${key.toLowerCase()};
        }
      ''';
    });

    return '''
      ${importList.join('\n')}

      class $className {
        // Singleton
        static $className? _instance;
        $className._internal();
        factory $className() => _getInstance();
        static $className get instance => _getInstance();
        static _getInstance() {
          if (_instance == null) _instance = $className._internal();
          return _instance;
        }

        $injectString
      }
    ''';
  }
}
